@@ -1,5 +1,6 @@
 # mypy: ignore-errors
 import os
+from collections import defaultdict
 from functools import lru_cache
 from typing import Optional, Union, Callable, List, Dict
 
@@ -65,6 +66,8 @@ def integration_test():
             finally:
                 _called_from_inside_tl_integration_test_decorator = False
 
+            leap_binder.check()
+
         return inner
 
     return decorating_function
@@ -230,9 +233,11 @@ def tensorleap_custom_metric(name: str,
                              direction: Union[MetricDirection, Dict[str, MetricDirection]] = MetricDirection.Downward,
                              compute_insights: Optional[Union[bool, Dict[str, bool]]] = None,
                              connects_to=None):
-    def decorating_function(user_function: Union[CustomCallableInterfaceMultiArgs,
-    CustomMultipleReturnCallableInterfaceMultiArgs,
-    ConfusionMatrixCallableInterfaceMultiArgs]):
+    name_to_unique_name = defaultdict(set)
+
+    def decorating_function(
+            user_function: Union[CustomCallableInterfaceMultiArgs, CustomMultipleReturnCallableInterfaceMultiArgs,
+            ConfusionMatrixCallableInterfaceMultiArgs]):
         for metric_handler in leap_binder.setup_container.metrics:
             if metric_handler.metric_handler_data.name == name:
                 raise Exception(f'Metric with name {name} already exists. '
@@ -352,6 +357,11 @@ def tensorleap_custom_metric(name: str,
 
             ordered_connections = [kwargs[n] for n in mapping_inner.arg_names if n in kwargs]
             ordered_connections = list(args) + ordered_connections
+
+            if user_unique_name in name_to_unique_name[mapping_inner.name]:
+                user_unique_name = f'{user_unique_name}_{len(name_to_unique_name[mapping_inner.name])}'
+            name_to_unique_name[mapping_inner.name].add(user_unique_name)
+
             _add_mapping_connection(user_unique_name, ordered_connections, mapping_inner.arg_names,
                                     mapping_inner.name, NodeMappingType.Metric)
 
@@ -374,6 +384,8 @@ def tensorleap_custom_metric(name: str,
 def tensorleap_custom_visualizer(name: str, visualizer_type: LeapDataType,
                                  heatmap_function: Optional[Callable[..., npt.NDArray[np.float32]]] = None,
                                  connects_to=None):
+    name_to_unique_name = defaultdict(set)
+
     def decorating_function(user_function: VisualizerCallableInterface):
         for viz_handler in leap_binder.setup_container.visualizers:
             if viz_handler.visualizer_handler_data.name == name:
@@ -450,6 +462,10 @@ def tensorleap_custom_visualizer(name: str, visualizer_type: LeapDataType,
             if 'user_unique_name' in kwargs:
                 user_unique_name = kwargs['user_unique_name']
 
+            if user_unique_name in name_to_unique_name[mapping_inner.name]:
+                user_unique_name = f'{user_unique_name}_{len(name_to_unique_name[mapping_inner.name])}'
+            name_to_unique_name[mapping_inner.name].add(user_unique_name)
+
             ordered_connections = [kwargs[n] for n in mapping_inner.arg_names if n in kwargs]
             ordered_connections = list(args) + ordered_connections
             _add_mapping_connection(user_unique_name, ordered_connections, mapping_inner.arg_names,
@@ -972,6 +988,8 @@ def tensorleap_gt_encoder(name: str):
 
 
 def tensorleap_custom_loss(name: str, connects_to=None):
+    name_to_unique_name = defaultdict(set)
+
     def decorating_function(user_function: CustomCallableInterface):
         for loss_handler in leap_binder.setup_container.custom_loss_handlers:
             if loss_handler.custom_loss_handler_data.name == name:
@@ -1040,6 +1058,10 @@ def tensorleap_custom_loss(name: str, connects_to=None):
             if 'user_unique_name' in kwargs:
                 user_unique_name = kwargs['user_unique_name']
 
+            if user_unique_name in name_to_unique_name[mapping_inner.name]:
+                user_unique_name = f'{user_unique_name}_{len(name_to_unique_name[mapping_inner.name])}'
+            name_to_unique_name[mapping_inner.name].add(user_unique_name)
+
             ordered_connections = [kwargs[n] for n in mapping_inner.arg_names if n in kwargs]
             ordered_connections = list(args) + ordered_connections
             _add_mapping_connection(user_unique_name, ordered_connections, mapping_inner.arg_names,
